/* Interface driver for the VE3IFB 8530 card (PI card) Copyright 1990 by Dave
   Perry, VE3IFB

   This version is designed for PMNOS.  It requires the instalation
   of the PI2DRVR.SYS  OS/2 physical device driver. Only one card
   is supported.



History Minor delinting - KA9Q 2/2/91 March 23 92 dp - Fixed bug in B port
   driver where ff in the data was not being handled correctly. - Fixed bug
   where receive packet would get clobbered if a new packet was enqueued for
   transmission. Greatly improves performance with sliding window protocols.
   (tdelay would overwrite contents of the interrupt control register in the
   scc). March 29,92 dp - Last release was based on old sources - corrected.
   Dec  '94      Converted to OS/2 Walt Corey

   Feb - Mar '95  Additional work done on
                  OS/2 conversion by Dale Heatherington, WA4DSY

   Version 1.0 released March 17 1995 by WA4DSY.


Portions of this driver were derived from the Eagle card driver by Art
   Goldman, WA3CVG. It has been very extensively modified from his work, due
   to the addition of DMA support and due to differences in the hardware.
   The PI card is NOT an Eagle card clone. It is an original design by Dave
   Perry, VE3IFB.  Art's copyright notice follows:

Written by Art Goldman, WA3CVG - (c) Copyright 1987 All Rights Reserved
   Permission for non-commercial use is hereby granted provided this notice
   is retained.  For info call: (301) 997-3838.

  **Note - All the hardware specfic code has been removed from this version.
           The above copyright may not apply.

*/
#define INCL_DOSDEVIOCTL
#define INCL_DOSPROCESS
#define INCL_DOSDEVICES
#define INCL_DOSFILEMGR
#define INCL_DOSERRORS
#include <os2.h>
#include <stdlib.h>

#include <time.h>
#include <stdio.h>
#include "global.h"
#include "mbuf.h"
#include "iface.h"

#include "pktdrvr.h"
#include "netuser.h"
#include "pi.h"
#include "ax25.h"
#include "trace.h"
#include "pc.h"

#include "session.h"
#include "lapb.h"
#include "proc.h"
#include "ip.h"
#include "devparam.h"
#include "cmdparse.h"


static int32 pi_ctl __ARGS ((struct iface * iface, int cmd, int set, int32 val));
static int pi_raw __ARGS ((struct iface * iface, struct mbuf * bp));
static int pi_stop __ARGS ((struct iface * iface));
static void reader (void *pv);


static struct pichan Pichan[2 * PIMAX];	/* channel table - 2 entries per card */
static int16 pinbr;

/*---------------------------------------------------------------------*/

APIRET SetPiParm(HFILE fh, UCHAR ubParm,long Value)
{
  //All functions are category 11

  return DosDevIOCtl(fh,11,ubParm,&Value,sizeof(long),NULL,NULL,0L,NULL);


}

/*-------------------------------------------------------------------*/
void GetPiInfo(int hdl, struct PiInfo *pi)
{ APIRET rc;
  ULONG datalen;

  datalen = sizeof(struct PiInfo );

  rc = DosDevIOCtl(hdl,11L,0x42L,NULL,0L,NULL,(void*)pi,datalen,&datalen);

 }

/*---------------------------------------------------------------------*/

/* Attach a PI interface to the system
 * argv[0]: hardware type, must be "pi"
 * argv[1]: I/O address, e.g., "0x300"    Not used in OS/2 version but must be supplied
 * argv[2]: vector, e.g., "2"		  ditto
 * argv[3]: dma channel (1..3)		  ditto
 * argv[4]: mode, must be:
 *          "ax25" (AX.25 UI frame format)
 * argv[5]: interface label, e.g., "pi0" or "pi0a$pi0b"
 * argv[6]: receiver packet buffer size in bytes
 * argv[7]: maximum transmission unit, bytes
 * argv[8]: channel A interface speed, e.g, "1200", 0 = ext. clock
 *          g suffix indicates G3RUH modem clocking, e.g, 9600g
 * argv[9]: channel B interface speed
 * argv[10]: First IP address, optional (defaults to Ip_addr);
 * argv[11]: Second IP address, optional (defaults to Ip_addr);
 */

int
pi_attach (int argc, char *argv[])
{
  unsigned long ulAction;
  TID tid;
  APIRET rc;
  struct mbuf *bp;
  extern void refiq ();
  register struct iface *if_pca, *if_pcb;
  struct pichan *hp;
  struct PiInfo pi;
  int dev;
  char i_state;
  char *separator, *ptr;
  int n;

  refiq ();			/* replenish interrupt buffer pool (in
				   mbuf.c) */
  /* Quick check to make sure args are good and mycall is set */
  if (strcmp ((char *) argv[4], (char *) "ax25") != 0)
    {
      tprintf ("PI: Mode %s unknown for interface %s\n",
	       argv[4], argv[5]);
      return -1;
    }
  if (if_lookup (argv[5]) != NULLIF)
    {
      tprintf ("PI: Interface %s already exists\n", argv[5]);
      return -1;
    }
  if (Mycall[0] == '\0')
    {
      tprintf ("PI: Set mycall first\n");
      return -1;
    }
 
  if (pinbr >= PIMAX)
    {
      tprintf ("PI: Maximum of %d PI cards supported\n", PIMAX);
      return -1;
    }
  dev = pinbr++;

 
  /* Create interface structures and fill in details */
  if_pca = (struct iface *) callocw (1, sizeof (struct iface));
  if_pcb = (struct iface *) callocw (1, sizeof (struct iface));

  if_pca->addr = if_pcb->addr = Ip_addr;
  if (argc > 10)
    if_pca->addr = resolve (argv[10]);
  if (argc > 11)
    if_pcb->addr = resolve (argv[11]);

  if (if_pca->addr == 0 || if_pcb->addr == 0)
    {
      tprintf ("PI: No IP address");
      free ((char *) if_pca);
      free ((char *) if_pcb);
      return -1;
    }



 if((separator = strchr(argv[5],'$')) != NULL) {  /* individual port names */
        *separator++ = 0; /* null terminate 1st name */
        if_pca->name = malloc((unsigned)strlen(argv[5])+1);
        strcpy(if_pca->name,argv[5]);
        if_pcb->name = malloc((unsigned)strlen(separator)+1);
        strcpy(if_pcb->name,separator);
    }
    else {
        /* Append "a" to interface associated with A channel */
        if_pca->name = malloc((unsigned)strlen(argv[5])+2);
        strcpy(if_pca->name,argv[5]);
        strcat(if_pca->name,"a");
        /* Append "b" to interface associated with B channel */
        if_pcb->name = malloc((unsigned)strlen(argv[5])+2);
        strcpy(if_pcb->name,argv[5]);
        strcat(if_pcb->name,"b");
    }
  

 
  if_pcb->mtu = if_pca->mtu = atoi (argv[7]);
  if_pcb->type = if_pca->type = CL_AX25;
  if_pcb->ioctl = if_pca->ioctl = pi_ctl;
  if_pca->dev = 2 * dev;	/* pi0a */
  if_pcb->dev = 2 * dev + 1;	/* pi0b */
  if_pcb->stop = if_pca->stop = pi_stop;
  if_pcb->output = if_pca->output = ax_output;
  if_pcb->raw = if_pca->raw = pi_raw;

  if (strcmp (argv[4], "ax25") == 0)
    {
      /* Must be true, was checked at top */
      if_pcb->send = if_pca->send = ax_send;
      if (if_pcb->hwaddr == NULLCHAR)
	if_pcb->hwaddr = mallocw (AXALEN);
      memcpy (if_pcb->hwaddr, Mycall, AXALEN);
      if (if_pca->hwaddr == NULLCHAR)
	if_pca->hwaddr = mallocw (AXALEN);
      memcpy (if_pca->hwaddr, Mycall, AXALEN);

   }

  setencap(if_pca,"AX25");
  setencap(if_pcb,"AX25");

 
  /* Link em in to the interface chain */
  if_pca->next = if_pcb;
  if_pcb->next = Ifaces;
  Ifaces = if_pca;
 

  /* set params in pichan table for CHANNEL B */
  hp = &Pichan[2 * dev + 1];	/* pi1 is offset 1 */
  hp->iface = if_pcb;
  
  if((ptr = strchr(argv[9],'g')) != NULL) {  /* use G3RUH clocking */
        *ptr = 0; /* get rid of the g so we can read the speed */
        hp->g3ruh = 1;
    } else hp->g3ruh = 0;

  hp->speed = (int16) atoi (argv[9]);
  hp->hFile = 0;
  hp->bufsiz = atoi (argv[6]);
   /* default channel access Params */
  hp->txdelay = 30;		/* 300 Ms */
  hp->persist = 128;		/* 50% persistence */
  hp->slotime = 30;		/* 300 Ms */
  hp->squeldelay = 3;		/* 30 Ms */
  hp->hFile = 0;

        rc = DosOpen (if_pcb->name,
		    (PHFILE) & hp->hFile,
		    (PULONG) & ulAction,
		    0L,
		    FILE_NORMAL,
		    FILE_OPEN,
		    OPEN_ACCESS_READWRITE | OPEN_SHARE_DENYREADWRITE,
		    0L);
      if (rc)
	{
	  tprintf ("%s returned %x on open\n", if_pcb->name, rc);
	  return (-1);
	}

	SetPiParm(hp->hFile,PI_TXDELAY,(ULONG)hp->txdelay);
	SetPiParm(hp->hFile,PI_PERSIST,(ULONG)hp->persist);
	SetPiParm(hp->hFile,PI_SLOTTIME,(ULONG)hp->slotime);
	SetPiParm(hp->hFile,PI_TAILTIME,(ULONG)hp->squeldelay);
	SetPiParm(hp->hFile,PI_SPEED,(ULONG)hp->speed);
	SetPiParm(hp->hFile,PI_CLK_MODE,(ULONG)hp->g3ruh);

/* Start channel B read thread */

      hp->tid = _beginthread (reader,
			  NULL,
			  4096,
			  (void *) hp);
  

  /* set params in pichan table for CHANNEL A */
  hp = &Pichan[2 * dev];	/* pi0a is offset 0 */
  hp->iface = if_pca;

  if((ptr = strchr(argv[8],'g')) != NULL) {  /* use G3RUH clocking */
        *ptr = 0; /* get rid of the g so we can read the speed */
        hp->g3ruh = 1;
    } else hp->g3ruh = 0;

  hp->speed = (int16) atoi (argv[8]);
  hp->bufsiz = atoi (argv[6]);
  /* default channel access Params */
  hp->txdelay = 15;		/* 15 mS */
  hp->persist = 128;		/* 50% persistence */
  hp->slotime = 15;		/* 15 mS */
  hp->squeldelay = 3;		/* 3 mS */
  hp->hFile = 0;


  rc = DosOpen (if_pca->name,
		(PHFILE) & hp->hFile,
		(PULONG) & ulAction,
		0L,
		FILE_NORMAL,
		FILE_OPEN,
		OPEN_ACCESS_READWRITE | OPEN_SHARE_DENYREADWRITE,
		0L);

  if (rc)
    {
      tprintf ("%s returned %x on open\n", if_pca->name, rc);
      return (-1);
    }

  	SetPiParm(hp->hFile,PI_TXDELAY,(ULONG)hp->txdelay);
	SetPiParm(hp->hFile,PI_PERSIST,(ULONG)hp->persist);
	SetPiParm(hp->hFile,PI_SLOTTIME,(ULONG)hp->slotime);
	SetPiParm(hp->hFile,PI_TAILTIME,(ULONG)hp->squeldelay);
	SetPiParm(hp->hFile,PI_SPEED,(ULONG)hp->speed);
	SetPiParm(hp->hFile,PI_CLK_MODE,(ULONG)hp->g3ruh);

/* Start channel A read thread */
	
  hp->tid = _beginthread (reader,
		      NULL,
		      4096,
		      (void *) hp);

  GetPiInfo(hp->hFile,&pi);

  tprintf("\nPI Card I/O Adr= 0x%03lx  IRQ= %ld  DMA= %ld  Bufsiz= %ld\n",
              pi.io_addr, pi.int_no, pi.dma_chan, pi.bufsiz);


  return 0;


}

/*-------------------------------------------------------------------------*/

/* Shut down interface */
int
pi_stop (iface)
     struct iface *iface;
{
  int16 dev,i;
  struct pichan *hp;
 
 if(iface->dev != 0) return;

 for(i=0;i<2;i++)

  {
   hp = &Pichan[i];	

       if(hp->tid) DosKillThread(hp->tid); 	 /* kill the reader thread */
       hp->tid = 0;
       if(hp->hFile) DosClose (hp->hFile);    /* Close device driver */
       hp->hFile = 0;
  }

  tprintf("PI: Interface stopped\n");
  return 0;
}
/*-----------------------------------------------------------------------*/

/* Send raw packet on pi card */

int
pi_raw (struct iface *iface, struct mbuf *bp)
{
  unsigned long ulBytes;
  char kickflag, *achBuffer;
  struct pichan *hp;

  dump (iface, IF_TRACE_OUT, CL_AX25, bp);
  iface->rawsndcnt++;
  iface->lastsent = secclock ();

  hp = &Pichan[iface->dev];
  achBuffer = malloc(hp->bufsiz);
  if (hp->hFile)
    {
      ulBytes = dqdata (bp, achBuffer, hp->bufsiz);
      DosWrite (hp->hFile, achBuffer, ulBytes, &ulBytes);
    }

  /* free_p(bp);  done in dqdata() */

  free(achBuffer);
  return 0;
}


/*------------------------------------------------------------------------*/
static void 
reader (void *pv)
{
  struct pichan *hp;
  struct mbuf *rcvbuf;
  struct mbuf *bp;
  struct phdr phdr;
  ULONG rc, ulBytes, ulAction, i_state;

  hp = (struct pichan *) pv;
  DosSetPrty(PRTYS_THREAD, PRTYC_FOREGROUNDSERVER, 0, 0);
  for (;;)
    {
      i_state = dirps ();
      rcvbuf = alloc_mbuf (hp->bufsiz + sizeof (struct phdr));
      restore (i_state);
      if (rcvbuf == NULLBUF)
	{
	  /* No memory, abort receiver */
	  tprintf ("PI: No memory available for receive buffers\n");
	  /* Restore original interrupt vector */
	  _endthread ();
	}

      rcvbuf->data += sizeof (struct phdr);
      rcvbuf->cnt = 0;

      rc = DosRead (hp->hFile, rcvbuf->data, (ULONG) hp->bufsiz, &ulBytes);
      if (rc == ERROR_INVALID_HANDLE)
	_endthread ();

      if ((rc == 0) && ulBytes)
	{
	  rcvbuf->cnt = ulBytes;
	  bp = pushdown (rcvbuf, sizeof (phdr));
	  phdr.type = CL_AX25;
	  phdr.iface = hp->iface;
	  memcpy (bp->data, (char *) &phdr, sizeof (phdr));

	  enqueue (&Hopper, rcvbuf);	/* queue it in */
	}
      else
	free_p (rcvbuf);
    }
}

/*----------------------------------------------------------------------*/

/* display PI Channel stats */
int
dopishow ()
{
  struct pichan *hp;
  struct PiInfo pi[2];
  int i;
  char *cp;
  
 
  hp = &Pichan[0];
  if(hp->hFile == 0) return -1;
  GetPiInfo(hp->hFile, &pi[0]);    //Get A channel PI info
  
  hp = &Pichan[1];
  if(hp->hFile == 0) return -1;
  GetPiInfo(hp->hFile, &pi[1]);    //Get B channel PI info


  tprintf("\nPMNOS PI card driver version:   %d.%02d\n",VERSION_MAJOR,VERSION_MINOR);
  tprintf("Physical Device Driver version: %d.%02d\n",(int)pi[0].Vmajor,
						     (int)pi[0].Vminor);
 
  tprintf ("PI Board Statistics:\n\n");
  tprintf ("Iface name Rxints  Txints  Exints  TxFrms  RxFrms  Crcerr  RxOvrs  TxUndr \n");
  tprintf ("---------  ------  ------  ------  ------  ------  ------  ------  ------ \n");

  for (i = 0; i < pinbr * 2; i++)
    {
      hp = &Pichan[i];
      cp = hp->iface->name;

      tprintf ("%8s % 8lu% 8lu% 8lu% 8u% 8u% 8u% 8u% 8u\n\n",
	       cp, pi[i].RxInts, pi[i].TxInts, pi[i].ExInts, pi[i].TXPackets,
	       pi[i].RXPackets, pi[i].CrcErr, pi[i].RxOvrs, pi[i].TxUndr);

 

    }

//for debuging queues in PDD...
 
   i = 0;

   tprintf("A free: %d  A RXQ: %d   A TXQ: %d  TXState %ld  RXState %ld\n",pi[i].freelist.nqueued,
						pi[i].rxqueue.nqueued,
						pi[i].txqueue.nqueued,
						pi[i].TxState,
						pi[i].RcvState);

   i = 1;

   tprintf("B free: %d  B RXQ: %d   B TXQ: %d  TXState %ld  RXState %ld\n",
						pi[i].freelist.nqueued,
						pi[i].rxqueue.nqueued,
						pi[i].txqueue.nqueued,
						pi[i].TxState,
						pi[i].RcvState);

  return 0;
}

/*-------------------------------------------------------------------------*/

static int
dopiclear(argc,argv,p)
int argc;
char *argv[];
void *p;
{
    struct pichan *hp;
    int i;
  
    for(i=0; i<pinbr*2; i++) 
	{
          hp = &Pichan[i];
          if(hp->hFile == 0) return -1;
          SetPiParm(hp->hFile,PI_CLR_CTRS,0L);
        }
    return 0;
}
/*--------------------------------------------------------------------*/
  
static struct cmds picmds[] = {
    "show",         dopishow,       0, 0, NULLCHAR,
    "clear",        dopiclear,      0, 0, NULLCHAR,
    NULLCHAR,
};
/*--------------------------------------------------------------------*/
int
dopistat(argc,argv,p)
int argc;
char *argv[];
void *p;
{
    return subcmd(picmds,argc,argv,p);
}
/*--------------------------------------------------------------------*/


/* Subroutine to set kiss params in channel tables */
int32
pi_ctl (iface, cmd, set, val)
     struct iface *iface;
     int cmd;
     int set;
     int32 val;
{
  struct pichan *hp;
  int32 t, ca;

  hp = &Pichan[iface->dev];	/* point to channel table */
  if(hp->hFile == 0) return -1;

  switch (cmd)
    {
    case PARAM_TXDELAY:
      if (set)
	{ hp->txdelay = val;
          SetPiParm(hp->hFile,PI_TXDELAY,(ULONG)val);
        }
      return hp->txdelay;

    case PARAM_PERSIST:
      if (set)
	{ hp->persist = val;
	  SetPiParm(hp->hFile,PI_PERSIST,(ULONG)val);
        }
        return uchar (hp->persist);

    case PARAM_SLOTTIME:
      if (set)
	{ hp->slotime = val;
	  SetPiParm(hp->hFile,PI_SLOTTIME,(ULONG)val);
        }
      return hp->slotime;

    case PARAM_TXTAIL:
      if (set)
       { hp->squeldelay = val;
	 SetPiParm(hp->hFile,PI_TAILTIME,(ULONG)val);
       }
      return hp->squeldelay;

    case PARAM_SPEED:           /* speed is param 10 */

        if (set) {
           hp->speed = val;
	   SetPiParm(hp->hFile,PI_SPEED,(ULONG)val);
	 } 
       return hp->speed;

    }
  return -1;
}
