
/* Header file for PI.C - OS/2 version by Dale Heatheringon WA4DSY  */


#include "global.h"
#define PIMAX	1		/* 1 cards max */

#define VERSION_MINOR 0
#define VERSION_MAJOR 1


struct pichan {
	int16 dev;		/* Device number */
	int16 bufsiz;		/* Size of rcvbuf */
	char rstate;		/* Set when !DCD goes to 0 (TRUE) */
	HFILE hFile;		/* handle to device */
	TID tid;		/* Thread ID of read function */
	int16 speed;		/* Line speed, bps */
	int16 g3ruh;		/* tapr 9600 modem clocking option */
	int16 txdelay;		/* Transmit Delay 10 ms/cnt */
	char persist;		/* Persistence (0-255) as a % */
	int16 slotime;		/* Delay to wait on persistence hit */
	int16 squeldelay;	/* Delay after XMTR OFF for squelch tail */
	struct iface *iface;	/* Associated interface */

#define IDLE	0		/* Transmitter off, no data pending */
#define ACTIVE	1		/* Transmitter on, sending data */
#define UNDERRUN 2		/* Transmitter on, flushing CRC */
#define FLAGOUT 3		/* CRC sent - attempt to start next frame */
#define DEFER 4 		/* Receive Active - DEFER Transmit */
#define ST_TXDELAY 5		/* Sending leading flags */
#define CRCOUT 6

#define RXERROR 2		/* Error -- Aborting current Frame */
#define RXABORT 3		/* ABORT sequence detected */
#define TOOBIG 4		/* too large a frame to store */

		};


/*----OS/2 PI device driver channel structure---------------------*/
/*
    This is a memory image of the device driver channel structure
    It can be read out of the driver by calling GetPiInfo().
*/

#pragma pack(1)

 struct  bfr {  USHORT addr;     // address of buffer
	  	USHORT nqueued;  // number of items queued
	  };

 struct PiInfo {
		  long int_no  	; // Hardware interrupt number
		  long io_addr	        ; // I/O address of PI card
		  long dma_chan	; // DMA channel
		  long speed		; // Baud rate; 0=external clock
		  long txdelayparm	; // TX Delay in milliseconds
		  long persist		 ; // Persistance probability out of 256
		  long slottime	 ; // Slot time for back off
		  long tailtime	 ; // Time from end of frame to drop RTS
		  long clkmode		 ; // Clocking mode
		  long bufsiz		 ; // DMA buffer size
		  long numbufs		 ; // Number of DMA buffers
		  long TXPackets  	 ; // Packets transmitted since driver OPENED
		  long RXPackets  	 ; // Packets received since driver OPENED
		  long DCDState	 ; // 1 = DCD true
		  long RTSState	 ; // 1 = RTS true
		  long RcvState	;
		  long TxState		;
		  long RxInts		 ; // receiver interrupt counter
		  long TxInts		 ; // transmitter interrupt counter
		  long ExInts		 ; // External interrupt counter
		  long CrcErr		 ; // number of rx CRC errors
		  long RxOvrs		 ; //  rx packets lost due to overruns
		  long TxUndr		 ; // tx underrrun errors
		  long reserved1	 ;
		  UCHAR Vminor	 ;	// minor version number of driver
		  UCHAR Vmajor;	// major verson number
		  USHORT reserved2;
		  UCHAR RDFlag		;//READ is pending
		  UCHAR WRFlag		;//WRITE is pending
		  UCHAR FlushFlag	;//Flush in progress
		  ULONG XferAddr	;//callers data transfer buffer addr
		  struct bfr rxbufptr	;//RX dma buffer
		  struct bfr rxqueue	;//head of RX queue
		  struct bfr txbufptr	;//TX dma buffer
		  struct bfr freelist	;//head of free list
		  struct bfr allocptr	;//last mem address used
		  struct bfr txqueue	;//head of TX queue
		  USHORT bufstart	;// offset starting address of DMA buffers
		  USHORT DmaMemSize	;//total size of dma memory for buffers
		  USHORT txlength	;//length of current packet being sent
		  UCHAR  tstate	;//transmitter state
		  UCHAR rstate		;//receiver state
		  USHORT txcnt		;//number of chars transmitted this frame
		  USHORT rxcnt		;//chars received this frame
		  USHORT rx_tc		;//recv. baud rate time constant
		  USHORT tx_tc		;//tx baud rate time constant
		  USHORT handle	;//file handle
		  USHORT opencounter	;//counts opens
		  UCHAR  R8530WRT[16]	;//16 scc write registers
		  UCHAR R8530RD[16]	;//16 scc read registers
		  USHORT watchdog	;//watch dog timer value
		  USHORT watchdog_ctr	;//watchdog counter

		};


/* IOCtl functions for PI card driver  */


const UCHAR PI_GET_ALL	= 0x42;	// Read all PI card parameters into PiInfo
const UCHAR PI_SPEED 	= 0x43;	// Set baud rate
const UCHAR PI_TXDELAY	= 0x44;	// Set TX delay in milliseconds
const UCHAR PI_PERSIST	= 0x45;	// Set persistance value 1 to 255
const UCHAR PI_SLOTTIME	= 0x46;	// Set slot time value in milliseconds
const UCHAR PI_TAILTIME = 0x47;		// Set tail time value in milliseconds
const UCHAR PI_CLR_CTRS = 0x48;		// Clear tx and rx packet counters
const UCHAR PI_CLK_MODE = 0x49;		// Set PI clock mode 0 or 1

// Note: The B channel times are in 10s of milliseconds

/*---------------------------------------------------------------------------------*/

#pragma pack()

